package cn.fw.dalaran.common.utils;

import cn.fw.dalaran.common.exception.BusinessException;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.BeanWrapper;
import org.springframework.beans.BeanWrapperImpl;
import org.springframework.scheduling.support.CronSequenceGenerator;

import java.io.UnsupportedEncodingException;
import java.lang.reflect.Array;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.text.DecimalFormat;
import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.function.BiConsumer;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @author Devin
 * Date: 2018/4/3
 * Time: 17:22
 * Description: 通用工具类
 */
public class PublicUtil {

    public static final String DATA_FORMAT = "yyyy-MM-dd";
    public static final String DATA_FORMAT_M = "yyyyMM";
    public static final String DATA_FORMAT_CLEAR = "yyyyMMdd";
    public static final String TIME_FORMAT = "yyyy-MM-dd HH:mm:ss";
    public static final String TIME_FORMAT_H = "yyyy-MM-dd HH:mm";
    public static final String TIME_FORMAT_S = "yyyy-MM-dd HH:mm:ss.sss";
    public static final String TIME_FORMAT_MH = "MM-dd HH:mm";
    public static final String TIME_FORMAT_SHORT = "HHmm";
    public final static String UNDERLINE = "_";

    /**
     * 还原html标签
     *
     * @param str
     * @return
     */
    public static String toShowHtml(String str) {
        if (isEmpty(str)) {
            return str;
        }
        String html = str;
        html = replace(html, "&amp;", "&");
        html = replace(html, "&lt;", "<");
        html = replace(html, "&gt;", ">");
        html = replace(html, "&quot;", "\"");
        html = replace(html, "<br>", "\n");
        return html;
    }

    /**
     * 判断包含字符串
     *
     * @param source
     * @param str
     * @return
     */
    public static Boolean containsStr(String source, String str) {
        return isNotEmpty(source) && source.contains(str);
    }

    /**
     * 获取字符串长度
     *
     * @param text
     * @return
     */
    public static int getLength(String text) {
        int textLength = text.length();
        int length = textLength;
        for (int i = 0; i < textLength; ++i) {
            if (String.valueOf(text.charAt(i)).getBytes().length > 1) {
                ++length;
            }
        }
        // 三元表达式, 如果length是偶数, 返回length/2, 否则返回(length/2) + 1
        return length % 2 == 0 ? length / 2 : length / 2 + 1;
    }

    /**
     * 截取字符串长度，并在字符串尾部添加省略号
     *
     * @param str
     * @param length
     * @return
     */
    public static String subMaxString(String str, Integer length) {
        if (isNotEmpty(str) && str.length() > length) {
            Integer realLength = 0;
            int len = str.length();
            for (int i = 0; i < len; ++i) {
                if (String.valueOf(str.charAt(i)).getBytes().length > 1) {
                    realLength = realLength + 2;
                } else {
                    realLength = realLength + 1;
                }
                if (realLength > length) {
                    str = toAppendStr(replaceBlank(str).substring(0, i), "...");
                    break;
                }
            }
        }
        return str;
    }

    /**
     * 替换空格符号
     *
     * @param str
     * @return
     */
    public static String replaceBlank(String str) {
        if (isEmpty(str)) {
            return "";
        }
        Pattern p = Pattern.compile("\\s*|\t|\r|\n");
        Matcher m = p.matcher(str);
        return m.replaceAll("");
    }

    /**
     * 替换字符串
     *
     * @param source
     * @param oldString
     * @param newString
     * @return
     */
    public static String replace(String source, String oldString, String newString) {
        if (!"".equals(source)) {
            StringBuffer output = new StringBuffer();
            int lengthOfSource = source.length();
            int lengthOfOld = oldString.length();
            int posStart;
            int pos;
            for (posStart = 0; (pos = source.indexOf(oldString, posStart)) >= 0; posStart = pos + lengthOfOld) {
                output.append(source, posStart, pos);
                output.append(newString);
            }
            if (posStart < lengthOfSource) {
                output.append(source.substring(posStart));
            }
            return output.toString();
        }
        return "";
    }

    /**
     * encode url
     *
     * @param url
     * @return
     */
    public static String getUrlEncode(String url) {
        try {
            url = URLEncoder.encode(url, "utf-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        return url;
    }

    /**
     * 替换所有字符串
     *
     * @param str
     * @param oldStr
     * @param newStr
     * @return
     */
    public static String replaceAll(String str, String oldStr, String newStr) {
        int length = str.indexOf(oldStr);
        int j = 0;
        for (String temp = ""; length != -1; length = str.indexOf(oldStr)) {
            ++j;
            if (j == 10) {
                break;
            }
            temp = str.substring(length + 1);
            str = str.substring(0, length);
            str = str + newStr + temp;
        }
        return str;
    }

    /**
     * 对象强制转换成int, 转换失败返回自己指定的默认值
     *
     * @param value
     * @param defaultValue
     * @return
     */
    public static int parseInt(Object value, int defaultValue) {
        try {
            return Integer.parseInt(String.valueOf(value));
        } catch (Exception var3) {
            return defaultValue;
        }
    }

    /**
     * 对象强制转换成long, 转换失败返回自己指定的默认值
     *
     * @param value
     * @param defaultValue
     * @return
     */
    public static long parseLong(Object value, long defaultValue) {
        try {
            return Long.parseLong(String.valueOf(value));
        } catch (Exception var3) {
            return defaultValue;
        }
    }

    /**
     * 转换成date
     *
     * @param date
     * @return
     */
    public static Date parseDate(Object date) {
        return parseDate(String.valueOf(date));
    }

    /**
     * 转换成date
     *
     * @param date
     * @return
     */
    public static Date parseDate(String date) {
        try {
            SimpleDateFormat ex = new SimpleDateFormat(TIME_FORMAT);
            return ex.parse(date);
        } catch (Exception var2) {
            return null;
        }
    }

    /**
     * str转换成date
     *
     * @param date
     * @param format
     * @return
     */
    public static Date parseDate(String date, String format) {
        try {
            SimpleDateFormat ex = new SimpleDateFormat(format);
            return ex.parse(date);
        } catch (Exception var3) {
            return null;
        }
    }

    /**
     * date转换成字符串
     * @param date
     * @return
     */
    public static String dateParseString(Date date) {
        date = Optional.ofNullable(date).orElse(new Date());
        return new SimpleDateFormat(TIME_FORMAT).format(date);
    }

    /**
     * 值解析
     *
     * @param value
     * @param defaultValue
     * @return
     */
    public static String parseString(Object value, String defaultValue) {
        return null != value && !"null".equals(String.valueOf(value).toLowerCase()) ? String.valueOf(value) : defaultValue;
    }

    /**
     * 当前时间
     *
     * @return
     */
    public static String getCurrentTime() {
        return getCurrentTime("yyyy-MM-dd HH:mm:ss");
    }

    /**
     * 当前时间
     *
     * @return
     */
    public static Date getCurrentDate() {
        return new Date();
    }

    /**
     * 按自己指定的格式获取当前时间字符串
     *
     * @param format
     * @return
     */
    public static String getCurrentTime(String format) {
        Date nowTime = new Date();
        SimpleDateFormat fmt = new SimpleDateFormat(format);
        return fmt.format(nowTime);
    }

    /**
     * 把指定的Date对象转换成自己指定的格式的字符串
     *
     * @param date
     * @param fmt
     * @return
     */
    public static String fmtDate(Date date, String fmt) {
        if (null == date) {
            return "";
        } else {
            SimpleDateFormat f = new SimpleDateFormat(fmt);
            return f.format(date);
        }
    }

    /**
     * 把指定的Date对象格式化成"yyyy-MM-dd HH:mm:ss"这种样式的字符串
     *
     * @param date
     * @return
     */
    public static String fmtDate(Date date) {
        if (null == date) {
            return "";
        } else {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            return sdf.format(date);
        }
    }

    /**
     * 转换为时间（天,时:分:秒.毫秒）
     *
     * @param timeMillis
     * @return
     */
    public static String formatDateTime(long timeMillis) {
        long day = timeMillis / (24 * 60 * 60 * 1000);
        long hour = (timeMillis / (60 * 60 * 1000) - day * 24);
        long min = ((timeMillis / (60 * 1000)) - day * 24 * 60 - hour * 60);
        long s = (timeMillis / 1000 - day * 24 * 60 * 60 - hour * 60 * 60 - min * 60);
        long sss = (timeMillis - day * 24 * 60 * 60 * 1000 - hour * 60 * 60 * 1000 - min * 60 * 1000 - s * 1000);
        return (day > 0 ? day + "," : "") + hour + ":" + min + ":" + s + "." + sss;
    }

    /**
     * uuid生成器
     *
     * @return
     */
    public static String nextId() {
        return UUID.randomUUID().toString().replaceAll("-", "");
    }

    /**
     * 生成固定格式uuid
     *
     * @return
     */
    public static String getUUID() {
        String uid = "0";
        SimpleDateFormat sf = new SimpleDateFormat("yyyyMMddHHmmssSSS");
        String tempId = sf.format(new Date());
        if (Long.parseLong(uid) >= Long.parseLong(tempId)) {
            uid = Long.parseLong(uid) + 1L + "";
        } else {
            uid = tempId;
        }
        return uid + getRandomString(10);
    }

    /**
     * 按指定的size, 随机生成长度为size的字符串
     *
     * @param size
     * @return
     */
    public static String getRandomString(int size) {
        char[] c = new char[]{'1', '2', '3', '4', '5', '6', '7', '8', '9', '0', 'q', 'w', 'e', 'r', 't', 'y', 'u', 'i', 'o', 'p', 'a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', 'z', 'x', 'c', 'v', 'b', 'n', 'm', 'J', 'V', 'A', 'E', 'L', 'Q', 'X', 'F', 'H', 'B', 'T', 'G', 'N', 'D', 'O', 'K', 'M', 'I', 'Y', 'U', 'W', 'C', 'Z', 'S', 'P', 'R'};
        Random random = new Random();
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < size; ++i) {
            sb.append(c[Math.abs(random.nextInt()) % c.length]);
        }
        return sb.toString();
    }

    /**
     * 随机小写数字字符串生成
     *
     * @param size
     * @return
     */
    public static String getRandomStringLittle(int size) {
        char[] c = new char[]{'1', '2', '3', '4', '5', '6', '7', '8', '9', '0', 'q', 'w', 'e', 'r', 't', 'y', 'u', 'i', 'o', 'p', 'a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', 'z', 'x', 'c', 'v', 'b', 'n', 'm'};
        Random random = new Random();
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < size; ++i) {
            sb.append(c[Math.abs(random.nextInt()) % c.length]);
        }
        return sb.toString();
    }

    /**
     * 随机数字字符串生成
     *
     * @param size
     * @return
     */
    public static String getRandomNumber(int size) {
        char[] c = new char[]{'1', '2', '3', '4', '5', '6', '7', '8', '9', '0'};
        Random random = new Random();
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < size; ++i) {
            sb.append(c[Math.abs(random.nextInt()) % c.length]);
        }
        return sb.toString();
    }

    /**
     * 非空判断
     *
     * @param obj
     * @return
     */
    public static boolean isNotEmpty(Object obj) {
        return !isEmpty(obj);
    }

    /**
     * 为空判断
     *
     * @param obj
     * @return
     */
    public static boolean isEmpty(Object obj) {
        if (obj == null) {
            return true;
        }
        if (obj instanceof String) {
            return "".equals(obj);
        }
        if (obj instanceof Collection) {
            Collection collection = (Collection) obj;
            return collection.size() == 0;
        }
        if (obj instanceof Map) {
            Map map = (Map) obj;
            return map.size() == 0;
        }
        return obj.getClass().isArray() && Array.getLength(obj) == 0;
    }

    /**
     * True断言处理
     *
     * @param flag
     */
    public static void assertTrue(Boolean flag) {
        assertTrue(flag, "");
    }

    /**
     * True断言处理
     *
     * @param flag
     * @param message
     */
    public static void assertTrue(Boolean flag, String message) {
        if (!flag) {
            throw new BusinessException(message);
        }
    }

    /**
     * False断言处理
     *
     * @param flag
     * @param message
     */
    public static void assertFalse(Boolean flag, String message) {
        assertTrue(!flag, message);
    }

    /**
     * 非空断言处理
     *
     * @param obj
     */
    public static void assertNotEmpty(Object obj) {
        assertNotEmpty(obj, "");
    }

    /**
     * 非空断言处理
     *
     * @param obj
     * @param message
     */
    public static void assertNotEmpty(Object obj, String message) {
        if (isEmpty(obj)) {
            throw new BusinessException(message);
        }
    }

    /**
     * 为空断言处理
     *
     * @param obj
     */
    public static void assertEmpty(Object obj) {
        assertEmpty(obj, "");
    }

    /**
     * 为空断言处理
     *
     * @param obj
     * @param message
     */
    public static void assertEmpty(Object obj, String message) {
        if (isNotEmpty(obj)) {
            throw new BusinessException(message);
        }
    }

    /**
     * 数据拼凑
     *
     * @param collection
     * @param delimiter
     * @param <E>
     * @return
     */
    public static <E> String collectionJoin(Collection<E> collection, String delimiter) {
        return collection.stream()
                .map(String::valueOf)
                .collect(Collectors.joining(delimiter));
    }

    /**
     * 获取随机字符串
     *
     * @return
     */
    public static String getNonceStr(int length) {
        String chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
        StringBuilder res = new StringBuilder();
        for (int i = 0; i < length; i++) {
            Random rd = new Random();
            res.append(chars.charAt(rd.nextInt(chars.length() - 1)));
        }
        return res.toString();
    }

    /**
     * 数字判断
     *
     * @param str
     * @return
     */
    public static boolean isDigitalString(String str) {
        if (isEmpty(str)) {
            return false;
        } else {
            try {
                Double.parseDouble(str);
                return true;
            } catch (Exception var2) {
                return false;
            }
        }
    }

    /**
     * 布尔值判断
     *
     * @param str
     * @return
     */
    public static boolean equalsFlag(Object str) {
        try {
            if ("true".equals(str)) {
                return true;
            }
        } catch (Exception var2) {
            var2.printStackTrace();
        }

        return false;
    }

    /**
     * 数字校验
     *
     * @param str
     * @return
     */
    public static boolean isNumeric(String str) {
        try {
            return str != null && str.matches("\\d*.?\\d*");
        } catch (Exception var2) {
            var2.printStackTrace();
            return false;
        }
    }

    /**
     * 字符串截取
     *
     * @param array
     * @param value
     * @return
     */
    public static int arrayIndexOf(Object array, Object value) {
        if (array != null && value != null) {
            int len = Array.getLength(array);

            for (int i = 0; i < len; ++i) {
                if (value.equals(Array.get(array, i))) {
                    return i;
                }
            }

            return -1;
        } else {
            return -1;
        }
    }

    /**
     * 字符串utf-8格式转换
     *
     * @param s
     * @return
     */
    public static String toUtf8String(String s) {
        StringBuilder sb = new StringBuilder();

        for (int i = 0; i < s.length(); ++i) {
            char c = s.charAt(i);
            if (c <= 255) {
                sb.append(c);
            } else {
                byte[] b;
                try {
                    b = Character.toString(c).getBytes("utf-8");
                } catch (Exception var7) {
                    var7.printStackTrace();
                    b = new byte[0];
                }
                for (byte aB : b) {
                    int k = aB;
                    if (k < 0) {
                        k += 256;
                    }
                    sb.append("%").append(Integer.toHexString(k).toUpperCase());
                }
            }
        }
        return sb.toString();
    }

    /**
     * 通用转换
     *
     * @param value
     * @return
     */
    public static String sicenToComm(Object value) {
        return value == null ? null : (value instanceof Double ? sicenToCommDouble((Double) value) : String.valueOf(value));
    }

    /**
     * 通用转换
     *
     * @param value
     * @return
     */
    public static String sicenToCommDouble(double value) {
        String retValue = null;
        DecimalFormat df = new DecimalFormat();
        df.setMinimumFractionDigits(0);
        df.setMaximumFractionDigits(5);
        retValue = df.format(value);
        retValue = retValue.replaceAll(",", "");
        return retValue;
    }

    /**
     * 都不转换成字符串
     *
     * @param value
     * @return
     */
    public static String roundStr(Double value) {
        DecimalFormat df = new DecimalFormat("#.00");
        return df.format(value);
    }

    /**
     * double转换
     *
     * @param value
     * @param count
     * @return
     */
    public static Double round(Double value, int count) {
        try {
            BigDecimal e = new BigDecimal(value);
            return e.setScale(count, 4).doubleValue();
        } catch (Exception var3) {
            return 0x0.0p0;
        }
    }


    /**
     * double转换
     *
     * @param value
     * @param count
     * @return
     */
    public static Double round(Object value, int count) {
        return round(parseDouble(value), count);
    }

    /**
     * 转换double
     *
     * @param value
     * @return
     */
    public static Double parseDouble(Object value) {
        return value == null ? Double.valueOf(0.0D) : parseDouble(String.valueOf(value));
    }

    /**
     * 转换double
     *
     * @param value
     * @return
     */
    public static Double parseDouble(String value) {
        return parseDouble(value, 0x0.0p0);
    }

    /**
     * 转换double，设置默认值
     *
     * @param value
     * @param dafualtVal
     * @return
     */
    public static Double parseDouble(String value, Double dafualtVal) {
        Double r = dafualtVal;
        try {
            if (value != null) {
                value = value.trim();
                if (value.endsWith("%")) {
                    value = value.substring(0, value.length() - 1);
                }
                r = Double.parseDouble(value);
            }
        } catch (Exception var4) {
            var4.printStackTrace();
        }
        return r;
    }

    /**
     * 首字母小写
     *
     * @param s
     * @return
     */
    public static String toLowerCaseFirstOne(String s) {
        return Character.isLowerCase(s.charAt(0)) ? s : Character.toLowerCase(s.charAt(0)) + s.substring(1);
    }

    /**
     * 首字母大写
     *
     * @param s
     * @return
     */
    public static String toUpperCaseFirstOne(String s) {
        return Character.isUpperCase(s.charAt(0)) ? s : Character.toUpperCase(s.charAt(0)) + s.substring(1);
    }

    /**
     * 追加拼凑
     *
     * @param strs
     * @return
     */
    public static String toAppendStr(Object... strs) {
        StringBuilder sb = new StringBuilder();
        Object[] var2 = strs;
        int var3 = strs.length;
        for (int var4 = 0; var4 < var3; ++var4) {
            Object str = var2[var4];
            if (isNotEmpty(str)) {
                sb.append(str);
            }
        }
        return sb.toString();
    }

    /**
     * 追加拼凑
     *
     * @param strs
     * @param split
     * @return
     */
    public static String toAppendStr(List<Object> strs, String split) {
        StringBuilder sb = new StringBuilder();
        Object[] var2 = strs.toArray();
        int var3 = strs.size();
        for (int var4 = 0; var4 < var3; ++var4) {
            Object str = var2[var4];
            if (isNotEmpty(str)) {
                if (var4 > 0) {
                    sb.append(split);
                }
                sb.append(str);
            }
        }
        return sb.toString();
    }

    /**
     * 正则格式匹配
     *
     * @param pattern
     * @param str
     * @return
     */
    public static boolean match(String pattern, String str) {
        Pattern p = Pattern.compile(pattern);
        Matcher m = p.matcher(str);
        return m.find();
    }

    /**
     * object对象转字符串
     *
     * @param obj
     * @return
     */
    public static String toStrString(Object obj) {
        return isNotEmpty(obj) ? String.valueOf(obj) : "";
    }

    /**
     * object对象转字符串 or Null
     *
     * @param obj
     * @return
     */
    public static String toStrStringNull(Object obj) {
        return isNotEmpty(obj) ? String.valueOf(obj) : null;
    }

    /**
     * 指定时间格式
     *
     * @param val
     * @param format
     * @return
     */
    public static Object fmtDate(ZonedDateTime val, String format) {
        return val == null ? null : val.format(DateTimeFormatter.ofPattern(format));
    }

    /**
     * 默认时间格式化
     *
     * @param val
     * @return
     */
    public static Object fmtDate(ZonedDateTime val) {
        return fmtDate(val, "yyyy-MM-dd HH:mm:ss");
    }

    /**
     * 随机字符串长度生成
     *
     * @param length
     * @return
     */
    public static String getRandomBsString(int length) {
        String str = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
        Random random = new Random();
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < length; i++) {
            int number = random.nextInt(62);
            sb.append(str.charAt(number));
        }
        return sb.toString();
    }

    /**
     * 区间值随机数组生成
     *
     * @param min
     * @param max
     * @param n
     * @return
     */
    public static int[] getRandomNumArray(int min, int max, int n) {
        if (n > (max - min + 1) || max < min) {
            return null;
        }
        int[] result = new int[n];
        int count = 0;
        while (count < n) {
            int num = (int) (Math.random() * (max - min)) + min;
            boolean flag = true;
            for (int j = 0; j < n; j++) {
                if (num == result[j]) {
                    flag = false;
                    break;
                }
            }
            if (flag) {
                result[count] = num;
                count++;
            }
        }
        return result;
    }

    /**
     * 将字符串进行sha1加密
     *
     * @param str 需要加密的字符串
     * @return 加密后的内容
     */
    public static String sha1(String str) {
        try {
            MessageDigest digest = MessageDigest.getInstance("SHA-1");
            digest.update(str.getBytes());
            byte[] messageDigest = digest.digest();
            // Create Hex String
            StringBuilder hexString = new StringBuilder();
            // 字节数组转换为 十六进制 数
            for (byte aMessageDigest : messageDigest) {
                String shaHex = Integer.toHexString(aMessageDigest & 0xFF);
                if (shaHex.length() < 2) {
                    hexString.append(0);
                }
                hexString.append(shaHex);
            }
            return hexString.toString();

        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        }
        return "";
    }

    /**
     * md5加密
     *
     * @param text
     * @return
     */
    public static String md5(String text) {
        try {
            MessageDigest md = MessageDigest.getInstance("MD5");
            byte[] bytes = md.digest(text.getBytes(StandardCharsets.UTF_8));
            return toHex(bytes);
        } catch (Exception e) {
            e.printStackTrace();
            return text;
        }
    }

    /**
     * hex加密
     *
     * @param bytes
     * @return
     */
    public static String toHex(byte[] bytes) {
        final char[] hexDigits = "0123456789ABCDEF".toCharArray();
        StringBuilder ret = new StringBuilder(bytes.length * 2);
        for (byte aByte : bytes) {
            ret.append(hexDigits[(aByte >> 4) & 0x0f]);
            ret.append(hexDigits[aByte & 0x0f]);
        }
        return ret.toString();
    }

    /**
     * 字符串判空
     *
     * @param cs
     * @return
     */
    public static boolean isBlank(CharSequence cs) {
        int strLen;
        if (cs != null && (strLen = cs.length()) != 0) {
            for (int i = 0; i < strLen; ++i) {
                if (!Character.isWhitespace(cs.charAt(i))) {
                    return false;
                }
            }
            return true;
        } else {
            return true;
        }
    }

    /**
     * 字符串非空判断
     *
     * @param cs
     * @return
     */
    public static boolean isNotBlank(CharSequence cs) {
        return !isBlank(cs);
    }

    /**
     * 迭代元素
     *
     * @param elements
     * @param action
     * @param <E>
     */
    public static <E> void forEach(Iterable<? extends E> elements, BiConsumer<Integer, ? super E> action) {
        Objects.requireNonNull(elements);
        Objects.requireNonNull(action);
        int index = 0;
        for (E element : elements) {
            action.accept(index++, element);
        }
    }

    /**
     * 隐藏电话号码
     *
     * @param phone
     * @return
     */
    public static String hidePhone(String phone) {
        if (PublicUtil.isNotEmpty(phone)) {
            if (phone.length() > 7) {
                return phone.substring(0, 3) + "****" + phone.substring(7);
            } else {
                return phone;
            }
        }
        return phone;
    }

    /**
     * 复制不覆盖（当目标对象不为空时）
     *
     * @param source           拷贝数据源
     * @param target           拷贝目标对象
     * @param ignoreProperties 忽略属性
     * @param <E>              数据源类泛型
     * @param <R>              目标类泛型
     * @return
     */
    public static <E, R> R copyNotConvert(E source, R target, String... ignoreProperties) {
        BeanUtils.copyProperties(source, target, getNotNullPropertyName(target, ignoreProperties));
        return target;
    }

    /**
     * 获取不为空对象数组
     *
     * @param target 目标类
     * @return
     */
    public static String[] getNotNullPropertyName(Object target, String... ignoreProperties) {
        final BeanWrapper src = new BeanWrapperImpl(target);
        java.beans.PropertyDescriptor[] pds = src.getPropertyDescriptors();
        Set<String> emptyNames = new HashSet<>();
        for (java.beans.PropertyDescriptor pd : pds) {
            Object srcValue = src.getPropertyValue(pd.getName());
            if (srcValue != null) emptyNames.add(pd.getName());
        }
        Collections.addAll(emptyNames, ignoreProperties);
        String[] result = new String[emptyNames.size()];
        return emptyNames.toArray(result);
    }

    /**
     * 对象覆盖
     *
     * @param source           拷贝数据源
     * @param target           拷贝目标对象
     * @param ignoreProperties 忽略属性
     * @param <E>              数据源类泛型
     * @param <R>              目标类泛型
     * @return
     */
    public static <E, R> R copy(E source, R target, String... ignoreProperties) {
        BeanUtils.copyProperties(source, target, ignoreProperties);
        return target;
    }

    /**
     * cglib对象复制
     *
     * @param source           拷贝数据源
     * @param targetClass      拷贝目标类
     * @param ignoreProperties 忽略属性
     * @param <E>              数据源类泛型
     * @param <R>              目标类泛型
     * @return
     */
    public static <E, R> R copy(E source, Class<R> targetClass, String... ignoreProperties) {
        R target = null;
        if (isNotEmpty(source)) {
            try {
                target = targetClass.newInstance();
                BeanUtils.copyProperties(source, target, ignoreProperties);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return target;
    }

    /**
     * cglib对象复制
     *
     * @param sourceList       拷贝数据源列表
     * @param targetClass      拷贝目标类
     * @param ignoreProperties 忽略属性
     * @param <E>              数据源类泛型
     * @param <R>              目标类泛型
     * @param <R>              目标类泛型
     * @return
     */
    public static <E, R> List<R> copyList(List<E> sourceList, Class<R> targetClass, String... ignoreProperties) {
        return Optional.ofNullable(sourceList)
                .orElse(new ArrayList<>())
                .stream()
                .map(e -> copy(e, targetClass, ignoreProperties))
                .collect(Collectors.toList());
    }

    /**
     * 奖项均匀分布
     *
     * @param rewardList
     * @param total
     * @param clazz
     * @param <T>
     * @return
     */
    public static <T> List<T> randomRewardList(List<T> rewardList, Integer total, Class<T> clazz) {
        List<T> resultList = new ArrayList<>();
        int rewardNum = rewardList.size();
        int leftNum = total - rewardNum;
        if (rewardNum >= total) {
            resultList.addAll(rewardList);
        } else {
            if (rewardNum >= leftNum) {
                int x = rewardNum / leftNum;
                int y = rewardNum % leftNum;
                for (int i = 0; i < leftNum; i++) {
                    if (i < y) {
                        resultList.addAll(rewardList.subList(i * (x + 1), (i + 1) * (x + 1)));
                    } else {
                        resultList.addAll(rewardList.subList(i * x + y, (i + 1) * x + y));
                    }
                    try {
                        resultList.add(clazz.newInstance());
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            } else {
                int x = leftNum / rewardNum;
                int y = leftNum % rewardNum;
                for (int i = 0; i < rewardNum; i++) {
                    if (i < y) {
                        for (int n = 0; n < (x + 1); n++) {
                            try {
                                resultList.add(clazz.newInstance());
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                        }
                    } else {
                        for (int n = 0; n < x; n++) {
                            try {
                                resultList.add(clazz.newInstance());
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                        }
                    }
                    resultList.addAll(rewardList.subList(i, (i + 1)));
                }
            }
        }
        return resultList;
    }

    /**
     * 根据属性去重
     *
     * @param key
     * @param <T>
     * @return
     */
    public static <T> Predicate<T> distinctByKey(final Function<? super T, ?> key) {
        final Map<Object, Boolean> seen = new ConcurrentHashMap<>();
        return t -> Objects.isNull(seen.putIfAbsent(key.apply(t), Boolean.TRUE));
    }

    /**
     * 判断字符串是否为数字
     *
     * @param str
     * @return
     */
    private static boolean isStrNum(String str) {
        Pattern pattern = Pattern.compile("[0-9]*");
        Matcher isNum = pattern.matcher(str);
        return isNum.matches();
    }

    /**
     * 判断字符串是否为日期格式
     *
     * @param strDate
     * @return
     */
    public static boolean isStrDate(String strDate) {
        Pattern pattern = Pattern.compile("^((\\d{2}(([02468][048])|([13579][26]))[\\-\\/\\s]?((((0?[13578])|(1[02]))[\\-\\/\\s]?((0?[1-9])|([1-2][0-9])|(3[01])))|(((0?[469])|(11))[\\-\\/\\s]?((0?[1-9])|([1-2][0-9])|(30)))|(0?2[\\-\\/\\s]?((0?[1-9])|([1-2][0-9])))))|(\\d{2}(([02468][1235679])|([13579][01345789]))[\\-\\/\\s]?((((0?[13578])|(1[02]))[\\-\\/\\s]?((0?[1-9])|([1-2][0-9])|(3[01])))|(((0?[469])|(11))[\\-\\/\\s]?((0?[1-9])|([1-2][0-9])|(30)))|(0?2[\\-\\/\\s]?((0?[1-9])|(1[0-9])|(2[0-8]))))))(\\s(((0?[0-9])|([1-2][0-3]))\\:([0-5]?[0-9])((\\s)|(\\:([0-5]?[0-9])))))?$");
        Matcher m = pattern.matcher(strDate);
        return m.matches();
    }

    /***
     * 下划线命名转为驼峰命名
     *
     * @param para
     *        下划线命名的字符串
     */

    public static String underlineToHump(String para) {
        StringBuilder result = new StringBuilder();
        String a[] = para.split(UNDERLINE);
        for (String s : a) {
            if (!para.contains(UNDERLINE)) {
                result.append(s);
                continue;
            }
            if (result.length() == 0) {
                result.append(s.toLowerCase());
            } else {
                result.append(s.substring(0, 1).toUpperCase());
                result.append(s.substring(1).toLowerCase());
            }
        }
        return result.toString();
    }

    /***
     * 驼峰命名转为下划线命名
     *
     * @param para
     *        驼峰命名的字符串
     */

    public static String humpToUnderline(String para) {
        StringBuilder sb = new StringBuilder(para);
        int temp = 0;//定位
        if (!para.contains(UNDERLINE)) {
            for (int i = 0; i < para.length(); i++) {
                if (Character.isUpperCase(para.charAt(i))) {
                    sb.insert(i + temp, UNDERLINE);
                    temp += 1;
                }
            }
        }
        return sb.toString().toLowerCase();
    }

    /**
     * 获取时间差
     *
     * @param date1
     * @param date2
     * @return
     */
    public static String diffTimeStr(Date date1, Date date2) {
        long nd = 1000 * 24 * 60 * 60;// 一天的毫秒数
        long nh = 1000 * 60 * 60;// 一小时的毫秒数
        long diff = Math.abs(date1.getTime() - date2.getTime());
        long day = diff / nd;// 计算差多少天
        long hour = diff % nd / nh + day * 24;// 计算差多少小时
        if (day > 0) {
            return day + "天" + (hour - day * 24) + "小时";
        } else {
            return hour + "小时";
        }
    }

    /**
     * 获取下一次执行时间
     *
     * @param corn
     * @return
     */
    public static Date getNextCornTime(String corn) {
        CronSequenceGenerator cronSequenceGenerator = new CronSequenceGenerator(corn);
        return cronSequenceGenerator.next(new Date());
    }

    /**
     * 字符串拼接
     *
     * @param pattern
     * @param arguments
     * @return
     */
    public static String formatKey(String pattern, Object... arguments) {
        MessageFormat temp = new MessageFormat(pattern);
        return temp.format(arguments);
    }

}

