package cn.fw.dalaran.server.task;

import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.rpc.attendance.AttendanceRpc;
import cn.fw.dalaran.rpc.attendance.dto.StaffWorkStatusDto;
import cn.fw.dalaran.service.data.AccountService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2022/4/13 15:00
 * @Description 每天12点检查昨天员工上班状态
 */
@Slf4j
@Component
@RequiredArgsConstructor
@SuppressWarnings("Duplicates")
public class CheckStaffWorkStatusTask {

    private final AccountService accountService;
    private final AttendanceRpc attendanceRpc;

    /**
     * 每天12点检查人员昨天工作状态
     */
    @Scheduled(cron = "0 00 12 * * *")
    public void checkStaffWorkStatus() {
        List<Account> list = accountService.lambdaQuery()
                .eq(Account::getAccountStatus, 0)
                .list();
        if (CollectionUtils.isEmpty(list))
            return;
        List<StaffWorkStatusDto> staffWorkStatus = attendanceRpc.getStaffWorkStatus(list.stream()
                        .map(Account::getUserId)
                        .collect(Collectors.toList()),
                System.currentTimeMillis() - 36 * 3600 * 1000L + 10 * 60 * 1000);
        if (CollectionUtils.isEmpty(staffWorkStatus))
            return;
        accountService.lambdaUpdate()
                .in(Account::getUserId, staffWorkStatus.stream()
                        .filter(item -> Objects.equals(item.getWorkStatus(), 1))
                        .map(StaffWorkStatusDto::getUserId)
                        .distinct()
                        .collect(Collectors.toList())
                )
                .set(Account::getAccountStatus, 1)
                .update();
    }

}
