package cn.fw.dalaran.service;

import com.dingtalk.api.DefaultDingTalkClient;
import com.dingtalk.api.DingTalkClient;
import com.dingtalk.api.request.OapiRobotSendRequest;
import com.dingtalk.api.response.OapiRobotSendResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.List;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2022/2/8 9:24
 * @Description 钉钉推送服务类
 */
@Slf4j
public class DingTalkService {

    public static final String TEXT = "text";// 文本消息
    //public static final String WEBHOOK = "https://oapi.dingtalk.com/robot/send?access_token=c8426f5ac4c67572782824d6fbe58c919cf6d89a3d3705fe9e4aeaf0c448794d";// webhook
    //public static final String SECRET = "SECeb091441134148a38251627597e4e9d57c5246146ac6c1228d2da4db44d5a7e8";// 密钥
    public static final String WEBHOOK = "https://oapi.dingtalk.com/robot/send?access_token=e4b93e75a58be3bf142ad91b6a4b0f99c404d6eb1115ac6d9d36333a66aeec3c";// webhook
    public static final String SECRET = "SECb0357db53bc981e5886b5fe6c41560ddaeb3dd0ba1b2528308fdc4908dabd668";// 密钥

    /**
     * 推送消息
     *
     * @param body 发送的消息体
     * @throws Exception
     */
    public static void sendMessage(SendMessageBody body) throws Exception {
        log.info("开始推送钉钉消息：" + body);
        Long timestamp = System.currentTimeMillis();
        String secret = body.getSecret();

        String stringToSign = timestamp + "\n" + secret;
        Mac mac = Mac.getInstance("HmacSHA256");
        mac.init(new SecretKeySpec(secret.getBytes(StandardCharsets.UTF_8), "HmacSHA256"));
        byte[] signData = mac.doFinal(stringToSign.getBytes(StandardCharsets.UTF_8));
        String sign = URLEncoder.encode(new String(Base64.encodeBase64(signData)), "UTF-8");

        DingTalkClient client = new DefaultDingTalkClient(body.getWebhook() + "&timestamp=" + timestamp + "&sign=" + sign);
        OapiRobotSendRequest request = new OapiRobotSendRequest();

        OapiRobotSendRequest.At at = new OapiRobotSendRequest.At();
        if (body.isAtAll() || body.getMobileList() == null || body.getMobileList().size() == 0) {
            at.setIsAtAll(true);// 推送所有人
        } else {
            at.setAtMobiles(body.getMobileList());
            at.setIsAtAll(false);// 推送指定用户
        }
        request.setAt(at);

        if (TEXT.equals(body.getMsgType())) {// 文本消息
            request.setMsgtype(TEXT);
            OapiRobotSendRequest.Text text = new OapiRobotSendRequest.Text();
            text.setContent(body.getText());
            request.setText(text);
        }

        OapiRobotSendResponse response = client.execute(request);
        log.info("钉钉推送返回结果：" + response);
    }

    /**
     * 给群里发送一条钉钉消息 @所有人
     *
     * @param message 消息内容
     * @throws Exception
     */
    public static void sendToEverybody(String message) throws Exception {
        sendMessage(SendMessageBody.builder()
                .isAtAll(true)
                .msgType(TEXT)
                .webhook(WEBHOOK)
                .secret(SECRET)
                .text(message)
                .build());
    }

    /**
     * 给群里发送一条钉钉消息 @指定的人
     *
     * @param message    消息内容
     * @param peopleList 指定的人手机号集合
     * @throws Exception
     */
    public static void sendToSpecified(String message, List<String> peopleList) throws Exception {
        sendMessage(SendMessageBody.builder()
                .isAtAll(false)
                .msgType(TEXT)
                .webhook(WEBHOOK)
                .secret(SECRET)
                .text(message)
                .mobileList(peopleList)
                .build());
    }
}
